/*************************************************************************
 * Copyright (c) 2004 Nick T Tate
 * 
 * This file is part of Tearless SDK.
 * 
 * Tearless SDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Tearless SDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Tearless SDK; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ************************************************************************/
/*
 * Common header file for Tearless API.
 * 
 * This file is included in every other Tearless header.
 * 
 * Author: Nick T Tate 2003-09-07
 * 
 * $Id: tlcommon.h 12 2004-10-21 11:51:11Z nicktt $
 */

#ifndef TEARLESS_COMMON_HEADER
#define TEARLESS_COMMON_HEADER

#ifdef __cplusplus
extern "C" {
#define TEARLESS_USE_CLASSES
#define EXTENDS(type) : type {
#else
#define EXTENDS(type) { type;
#endif

#pragma pack (push, 1);

// Generic pointer
typedef void __far* PTR;

/* The types BYTE, WORD and DWORD are used to indicate data
 * of 8, 16 or 32 bits respectively, when no further information
 * about their meaning is known (or documented). If (and only if)
 * more specific information is known, use a more specific type.
 * Do not use char for anything but real ASCII characters.
 */
typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef unsigned long DWORD;

typedef unsigned char ubyte;
typedef unsigned short ushort;
typedef unsigned long ulong;

typedef signed char sbyte;

/* For BOOL, only the values 0 and 1 are guaranteed to work.
 * Defensively, one should write a WORD 0 or 1 (FALSE or TRUE),
 * but when reading only test the BYTE for zero or nonzero.
 * An alternative would be to use more specialized types ...
 */
typedef unsigned char BOOL;

/* Use this with caution. Only the values 0 and 1 (that is,
 * FALSE or TRUE) are allowed!
*/
typedef unsigned short WBOOL;

// TODO: Undefine if defined instead?
#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif

// Return types
 
/* An error code using the standard error codes.
 * 
 * (This definition may change to a bitfield, see
 * ERR_DETAIL_MASK and the enum ErrCodes below.)
 */
typedef unsigned short ERR;

/* For return value, ZERO means that AX always seems to be zero,
 * so either void or ERR could in principle be used, but there
 * is no point in testing for errors.
 */
typedef void ZERO; 

/* Not yet documented return value type.
 */
typedef void UNK;


// Common types
typedef unsigned long COLOR;

typedef struct {
	ushort width, height;
} Dimension;

typedef struct {
	ushort x, y;
} Position;

typedef struct Palette {
	WORD colors[16];
} Palette;

// Returned err codes must be ANDed with this mask to match the enum below.
#define ERR_DETAIL_MASK = 0xFFF

enum ErrCodes {
	ERR_NONE = 0,
	ERR_INACTIVITY_TIMEOUT = 2,
	ERR_7 = 7,
	ERR_8 = 8,
	ERR_9 = 9,
	ERR_10 = 0x10,
	// Maybe
	ERR_QUEUE_FULL = 0x12,
	ERR_OUT_OF_MEM = 0x15,
	// Maybe
	ERR_BUF_FULL = 0x17,
	// Maybe
	ERR_LENS_JAM = 0x18,
	ERR_INVALID_ARG = 0x20,
	ERR_BUF_NOT_ALLOCATED = 0x22,
	ERR_DISK_INACCESSIBLE = 0x30,
	ERR_DISK_FAILURE = 0x31,
	ERR_TIMEOUT = 0x32,
	ERR_NAMING_FAILURE = 0x33,
	ERR_NULL_PTR = 0x34,
	ERR_FILE_NOT_FOUND = 0x40,
	// Maybe
	ERR_READ_ONLY_OR_PTR_NOT_NULL = 0x41,
	ERR_43 = 0x43,
	ERR_44 = 0x44,
	ERR_45 = 0x45,
	ERR_DOS_ERR = 0x50,
	ERR_ACCESS_DENIED = 0x51,
	ERR_INVALID_FILE_HANDLE = 0x52,
	ERR_TOO_MANY_ARGS = 0x53,
	ERR_INVALID_ARG_DOS = 0x54,
	ERR_FILE_EXISTS = 0x55,
	ERR_TOO_MANY_OPEN_FILES = 0x56,
	ERR_FILE_NOT_FOUND_DOS = 0x57,
	ERR_CROSS_DEVICE_LINK = 0x58,
	ERR_RESULT_TOO_LARGE = 0x59,
	ERR_DISK_FULL = 0x5A,
	ERR_OUT_OF_MEM_DOS = 0x5B,
	ERR_DEADLOCK = 0x5C,
	ERR_BATT_DOOR_OPEN = 0x403,
	ERR_BATT_DEAD = 0x404,
	ERR_CF_DOOR_OPEN = 0x405,
	ERR_406 = 0x406
};

// Calling convention for all library functions and callbacks
#define TLCALL __pascal

// Find libraries
void __far * getTearlessLib(unsigned char libNum);
#pragma aux getTearlessLib = \
	"int 0FFh" \
	parm [ah] \
	value [ds dx] \
	modify [ds dx];

extern void loadTearlessLibs(void);
extern PTR TEARLESS_LIBS[16];

#pragma pack (pop);
#ifdef __cplusplus
}
#endif

#endif
