/*************************************************************************
 * Copyright (c) 2004 Nick T Tate
 * 
 * This file is part of Tearless SDK.
 * 
 * Tearless SDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Tearless SDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Tearless SDK; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ************************************************************************/
/*
 * Header file for Tearless Disk library (Lib 5).
 *
 * Author: Nick T Tate 2003-09-07
 * 
 * $Id: tldisk.h 12 2004-10-21 11:51:11Z nicktt $
 */

#ifndef TEARLESS_DISK_HEADER
#define TEARLESS_DISK_HEADER

#include "tlcommon.h"

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack (push, 1);

// TODO
typedef void DiskFreeInfo;

// Disk lib (complete except readDir and read/write)
struct TearlessDiskLib {
	ERR (TLCALL *getFreeDiskSpace)(DiskFreeInfo* infoStruct);
	char (TLCALL *getFlashCardDriveLetter)();
	ERR (TLCALL *openExistingFile)(char* absPath, WORD flags, DWORD* handle);
	ERR (TLCALL *createExclusiveFile)(char* absPath, WORD flags, DWORD* handle);
	ERR (TLCALL *closeFile)(DWORD handle);
	ERR (TLCALL *writeToFile)();
	ERR (TLCALL *readFromFile)();
	ERR (TLCALL *seek)(DWORD handle, DWORD method, DWORD offset);
	ERR (TLCALL *getFilePos)(DWORD handle, DWORD* pos);
	ERR (TLCALL *deleteFile)(char* absPath);
	ERR (TLCALL *doesFileOrDirExist)(char* absPath);
	ERR (TLCALL *getFileLen)(DWORD handle, DWORD* length);
	ERR (TLCALL *getFileAttribs)(char* absPath, DWORD* attributes);
	ERR (TLCALL *setFileAttribs)(char* absPath, DWORD attributes);
	ERR (TLCALL *rename)(char* oldName, char* newName);
	ERR (TLCALL *setFileTimeStamp)(char* fileName, DWORD dosTime);
	DWORD (TLCALL *getFileTimeStamp)(char* absPath);

	/*
	 * NOTE: In DOS, the file attribute is only a WORD so this function
	 * actually only returns a WORD and the functions taking attributes
	 * as arguments only care about the low WORD. But for API consistency,
	 * and if the underlying OS would ever change, we use DWORDs.
	 */
	DWORD (TLCALL *getReadOnlyDirAttribs)(DWORD dummy1, DWORD dummy2);
	BOOL (TLCALL *isReadOnly)(DWORD fileAttributes);
	BOOL (TLCALL *isDir)(DWORD fileAttributes);
	ERR (TLCALL *createDir)(char* absPath);
	ERR (TLCALL *deleteDir)(char* absPath);
	ERR (TLCALL *readDir)();
	DWORD (TLCALL *dosTimeToCTime)(DWORD dosTime);
	ERR (TLCALL *openCreateFile)(char* absPath, WORD flags, DWORD* handle);
};

#define DISK (*((struct TearlessDiskLib*)(TEARLESS_LIBS[5])))

#pragma pack (pop);
#ifdef __cplusplus
}
#endif

#endif

