/*************************************************************************
 * Copyright (c) 2004 Nick T Tate
 * 
 * This file is part of Tearless SDK.
 * 
 * Tearless SDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Tearless SDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Tearless SDK; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ************************************************************************/
/*
 * Header file for Tearless GUI library (Lib 8).
 *
 * Author: Nick T Tate 2003-09-07
 * 
 * $Id: tlgui.h 14 2004-10-21 14:41:29Z nicktt $
 */

#ifndef TEARLESS_GUI_HEADER
#define TEARLESS_GUI_HEADER

#include "tlcommon.h"
#include "tlwidget.h"

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack (push, 1);

// Enums

// TODO: Create other type with bitfields?
typedef enum Msg {
	MSG_NONE = 0,
	MSG_INIT = 1,
	MSG_CLEANUP	= 2,
	MSG_KEY_RIGHT = 3,
	MSG_KEY_LEFT = 4,
	// NOTE: Maybe swap up and down
 	MSG_KEY_DOWN = 5,
	MSG_KEY_UP = 6,
	MSG_KEY_MENU = 7,
	MSG_KEY_SET = 8,
	MSG_KEY_DISP = 9,
	MSG_KEY_ZOOM_OUT = 0xA,
	MSG_KEY_ZOOM_IN = 0xB,
	MSG_C = 0xC,
	MSG_VIDEO_CABLE_CHANGED = 0x12,
	MSG_RELEASE_SET = 0x13,
	MSG_RELEASE_RIGHT = 0x14,
	MSG_RELEASE_LEFT = 0x15,
	// Maybe
 	MSG_SET_HELD_THEN_RIGHT = 0x1F,
	// Maybe
 	MSG_SET_HELD_THEN_LEFT = 0x20,
	// Maybe
	MSG_SET_HELD_THEN_MENU_OR_RES20 = 0x21,
	MSG_RELEASE_SHUTTER_HALF = 0x22,
	MSG_SHUTTER_HELD_THEN_SET = 0x26,
	MSG_BATT_LOW = 0x27,
	MSG_30 = 0x30,
	MSG_KEYS_LEFT_RIGHT_DISP = 0x32,
	MSG_KEYS_RIGHT_SET_DISP = 0x33,
	MSG_100 = 0x100,
	MSG_101 = 0x101,
	MSG_102 = 0x102,
	MSG_103 = 0x103,
	MSG_104 = 0x104,
	MSG_106 = 0x106,
	MSG_109 = 0x109,
	MSG_10A = 0x10A,
	MSG_10B = 0x10B,
	MSG_10C = 0x10C,
	MSG_10D = 0x10D,
	MSG_10E = 0x10E,
	MSG_10F = 0x10F,
	MSG_110 = 0x110,
	MSG_MASK = 0x1FFF,
	MSG_FLAG_RELEASED = 0x2000,
	MSG_FLAG_ALONE = 0x4000,
	MSG_FLAG_SILENT = 0x8000
} Msg;

// Types
typedef struct MsgEnv {
	WBOOL* keepRunning;
	WORD msg;	// Msg
	ushort* requestMode;
	ushort request;
} MsgEnv;

typedef ERR TLCALL (*MsgHandler)(MsgEnv*);

typedef BYTE TLCALL (*MsgFlagsHandler)(Msg);

typedef struct Button {
	BYTE reserved_0;
	BYTE reserved_1;
	BYTE widgetType;	// WidgetType
	Widget widget;
	struct Button* childButton;
	MsgHandler msgHandler;

#ifdef TEARLESS_USE_CLASSES
	void draw(void);
	void setStatus(BYTE status);
	void setPos(short x, short y);
	void setSize(short width, short height);
	void setData(void* data);
	void setLeftPadding(short textPos);
	void getStatus(BYTE* status);
	void getData(void** dataHandlePtr);
	void getSize(ushort* width, ushort* height);
	void getPos(ushort* x, ushort* y);
#endif
} Button;

typedef struct TitleLabel EXTENDS(Label)
	BOOL underLineTitle;
	Button* titlebutton;

#ifdef TEARLESS_USE_CLASSES
	void draw(void);
#endif
} TitleLabel;

/* When allocated, form buttons default to FancyWidget in
 * image mode with STYLE_BORDER, unless noted otherwise here.
 * The default size and distribution of buttons also varies
 * with form type and number of buttons.
 */
typedef enum FormType {
	FORM_0 = 0,
	// Defaults to STYLE_BOX buttons
	FORM_1 = 1,
	// Defaults to STYLE_BOX_3D text buttons
	FORM_OK_CANCEL = 2,
	FORM_TIME = 3,
	// Defaults to STYLE_BOX_3D text buttons
	FORM_LANG = 4,
	// Defaults to TabWidget (image) buttons
	// TLS100: Broken
	FORM_TABBED = 5,
	FORM_6 = 6,
	FORM_7 = 7,
	FORM_SUB_SETTINGS = 8,
	FORM_9 = 9,
	// Defaults to STYLE_ICON_ALT buttons
	FORM_SETTINGS = 10,
	FORM_CUSTOM_MSG_HANDLER = 255
} FormType;


typedef struct Form {
	// Maybe
	WORD id;
	BYTE type;	// FormType
	ubyte numButtons;
	ubyte selBtnNum;
	Palette* palette;
	TitleLabel title;
	Button* buttons;
	struct Form** buttonForms;
	struct Form* parent;
	MsgHandler extraMsgHandler;
	MsgHandler msgHandler;
	MsgFlagsHandler msgFlagsHandler;

#ifdef TEARLESS_USE_CLASSES
	void setTitle(char* text, WORD y, WORD x, COLOR color);
	ERR free(void);
	void draw(void);
	void moveSelLeft(void);
	void moveSelRight(void);
	void moveSelUp(void);
	void moveSelDown(void);
	ERR passMsgToCustomHandler(MsgEnv* msgEnv);
	ERR passMsgToSelectedButton(MsgEnv* msgEnv);
	BYTE nextLeftEnabledNum(void);
	BYTE nextRightEnabledNum(void);
	Form* activate(void);
#endif
} Form;

// GUI lib
struct TearlessGuiLib {
	void (TLCALL *drawButton)(Button* firstButton);
	void (TLCALL *setButtonStatus)(Button* button, BYTE status);

	/* The value -1 for either x or y means that coordinate is unchanged.
	 */
	void (TLCALL *setButtonPos)(Button* button, short x, short y);

	/* The value -1 for either width or height means that dimension is unchanged.
	 */
	void (TLCALL *setButtonSize)(Button* button, short width, short height);
	void (TLCALL *setButtonData)(Button* button, void* data);
	void (TLCALL *setButtonLeftPadding)(Button* button, short textPos);
	void (TLCALL *getButtonStatus)(Button* button, BYTE* status);
	void (TLCALL *getButtonData)(Button* button, void** dataHandlePtr);
	void (TLCALL *getButtonSize)(Button* button, ushort* width, ushort* height);
	void (TLCALL *getButtonPos)(Button* button, ushort* x, ushort* y);
	void (TLCALL *drawTitleLabel)(TitleLabel* label);
	void (TLCALL *setFormTitle)(Form* form, char* text, ushort y, ushort x, COLOR color);

	/* See FormType for info on default buttons. The MsgHandler for
	 * a certain form type may make assumptions on the type of buttons.
	 * If unsure, use a custom MsgHandler (type FORM_CUSTOM_MSG_HANDLER).
	 */
	Form* (TLCALL *allocForm)(FormType type, ubyte numButtons);
	ERR (TLCALL *freeForm)(Form* form);
	void (TLCALL *drawForm)(Form* form);
	UNK (TLCALL *func3C)();
	UNK (TLCALL *func40)();
	void (TLCALL *moveFormSelLeft)(Form* form);
	void (TLCALL *moveFormSelRight)(Form* form);
	void (TLCALL *moveFormSelUp)(Form* form);
	void (TLCALL *moveFormSelDown)(Form* form);
	UNK (TLCALL *func54)();
	ERR (TLCALL *passMsgToCustomFormHandler)(Form* form, MsgEnv* msgEnv);
	ERR (TLCALL *passMsgToSelectedButton)(Form* form, MsgEnv* msgEnv);
	BYTE (TLCALL *nextLeftEnabledNum)(Form* form);
	BYTE (TLCALL *nextRightEnabledNum)(Form* form);
	Form* (TLCALL *setActiveForm)(Form* form);
	Form* (TLCALL *getActiveForm)(Form** formHandlePtr);
	MsgHandler (TLCALL *setSingleMsgHandler)(MsgHandler handler);

	/* This function wraps the msg in an MsgEnv, asks the active
	 * form for flags that controls whether to emit key sounds
	 * and update the battery low icon for this message.
	 * 
	 * On MSG_INIT it adjusts the position of the battery low icon
	 * according to video mode, and clears the fg page.
	 * 
	 * The MsgEnv is then passed to the MsgHandler for the active
	 * form which is form type dependant. The MsgEnv may then be passed
	 * around to various handlers (which may even alter the message)
	 * until it finally arrives back. If keepRunning has been set
	 * to false the palette will be restored and the fg page cleared.
	 * 
	 * If enabled by the flags, the battery low icon is then updated,
	 * if needed. In case of actual update, showPage(0) is called.
	 */
	ERR (TLCALL *sendFormMsg)(Msg msg, WBOOL* keepRunning);
	// Maybe
	int (TLCALL *requestPlayOrShootMode)(WORD play);
};

#define GUI (*((struct TearlessGuiLib*)(TEARLESS_LIBS[8])))

#ifdef TEARLESS_USE_CLASSES
/*
 * Inline wrappers for instance methods defined in library.
 */

	// Button
	inline void Button::draw(void) { GUI.drawButton(this); };
	inline void Button::setStatus(BYTE status) {
		GUI.setButtonStatus(this, status);
	};
	inline void Button::setPos(short x, short y) {
		GUI.setButtonPos(this, x, y);
	};
	inline void Button::setSize(short width, short height) {
		GUI.setButtonSize(this, width, height);
	};
	inline void Button::setData(void* data) {
		GUI.setButtonData(this, data);
	};
	inline void Button::setLeftPadding(short textPos) {
		GUI.setButtonLeftPadding(this, textPos);
	};
	inline void Button::getStatus(BYTE* status) {
		GUI.getButtonStatus(this, status);
	};
	inline void Button::getData(void** dataHandlePtr) {
		GUI.getButtonData(this, dataHandlePtr);
	};
	inline void Button::getSize(ushort* width, ushort* height) {
		GUI.getButtonSize(this, width, height);
	};
	inline void Button::getPos(ushort* x, ushort* y) {
		GUI.getButtonPos(this, x, y);
	};

	// TitleLabel
	inline void TitleLabel::draw(void) { GUI.drawTitleLabel(this); }

	// Form
	inline void Form::setTitle(char* text, ushort y, ushort x, COLOR color) {
		GUI.setFormTitle(this, text, y, x, color);
	};
	inline ERR Form::free(void) { return GUI.freeForm(this); };
	inline void Form::draw(void) { GUI.drawForm(this); };
	inline void Form::moveSelLeft(void) { GUI.moveFormSelLeft(this); };
	inline void Form::moveSelRight(void) { GUI.moveFormSelRight(this); };
	inline void Form::moveSelUp(void) { GUI.moveFormSelUp(this); };
	inline void Form::moveSelDown(void) { GUI.moveFormSelDown(this); };
	inline ERR Form::passMsgToCustomHandler(MsgEnv* msgEnv) {
		return GUI.passMsgToCustomFormHandler(this, msgEnv);
	};
	inline ERR Form::passMsgToSelectedButton(MsgEnv* msgEnv) {
		return GUI.passMsgToSelectedButton(this, msgEnv);
	};
	inline BYTE Form::nextLeftEnabledNum(void) {
		return GUI.nextLeftEnabledNum(this);
	};
	inline BYTE Form::nextRightEnabledNum(void) {
		return GUI.nextRightEnabledNum(this);
	};
	inline Form* Form::activate(void) {
		return GUI.setActiveForm(this);
	};
#endif

#pragma pack (pop);
#ifdef __cplusplus
}
#endif

#endif

