/*************************************************************************
 * Copyright (c) 2004 Nick T Tate
 * 
 * This file is part of Tearless SDK.
 * 
 * Tearless SDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Tearless SDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Tearless SDK; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ************************************************************************/
/*
 * Header file for Tearless System library.
 *
 * Author: Nick T Tate 2003-09-07
 * 
 * $Id: tlsys.h 12 2004-10-21 11:51:11Z nicktt $
 */

#ifndef TEARLESS_SYS_HEADER
#define TEARLESS_SYS_HEADER

#include "tlcommon.h"

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack (push, 1);

/*
 * Enums
 */
 
 /* Unfortunately, enums cannot be larger than 16 bits
  * in 16-bit (DOS) mode. At least not in OpenWatcom 1.2.
  * So the remaining values has been defined with macros.
  * Alternatively, Hi and Lo enum could be used.
  */
typedef enum KeyStates {
	FLAG_SHUTTER_HALF = 1,
	FLAG_SHUTTER_FULL = 2,
	FLAG_CF_DOOR_OPEN = 4,
	FLAG_SET = 8,
	FLAG_MENU = 0x10,
	FLAG_USB_CABLE_RELATED = 0x20,
	FLAG_DISP = 0x40,
	FLAG_RESERVED_400 = 0x400,
	FLAG_RESERVED_800 = 0x800,
	FLAG_ZOOM_OUT = 0x1000,
	FLAG_ZOOM_IN = 0x2000,
	FLAG_BATT_DOOR_OPEN = 0x4000,
	FLAG_VIDEO_CABLE_ATTACHED = 0x8000
#define FLAG_USB_CONNECTED 0x10000
#define FLAG_RIGHT 0x20000
#define FLAG_LEFT 0x40000
#define FLAG_POWER 0x80000
#define FLAG_PLAY_MODE 0x100000
} KeyStates;

// Duplicate for now
typedef enum KeyStates KeyStatesLo;

// Duplicate for now
typedef enum KeyStatesHi {
	HI_USB_CONNECTED = 1,
	HI_RIGHT = 2,
	HI_LEFT = 4,
	HI_POWER = 8,
	HI_PLAY_MODE = 0x10
} KeyStatesHi;

typedef enum RunLevel {
	rlShooting   = 3,
	rlPowerDown  = 4,
	rlPlay       = 5,
	rlUsbStorage = 7
} RunLevel;

typedef enum HalEventType {
	runLevelChange = 3,
	keyStateChange = 4,
	weakBattChange = 5
} HalEventType;

/*
 * Types
 */

typedef struct {
	WORD type;	// HalEventType
} HalEvent;

typedef struct RunLevelEvent EXTENDS(HalEvent)
	BYTE reserved_2;
	WORD reserved_3;
	WORD newRunLevel; // RunLevel
	WORD reserved_7;
} RunLevelEvent;

typedef struct KeyStateEvent EXTENDS(HalEvent)
	BYTE reserved_2;
	WORD reserved_3;
	DWORD keyStates;
} KeyStateEvent;

typedef struct WeakBattEvent EXTENDS(HalEvent)
	BYTE reserved_2;
	WORD reserved_3;
	WORD config;
	WORD minTimeLeft;
} WeakBattEvent;

typedef ERR TLCALL (*HalEventHandler)(HalEvent*, void*);

// TODO
typedef void SysInfo;
// TODO
typedef void PackedTimeStruct;

/*
 * System lib
 */

struct TearlessSysLib {
	SysInfo* (TLCALL *getSysInfo)(void);
	// Maybe
	DWORD (TLCALL *getFwVersion)(void);
	
	/*
	 * This function processes all queued HAL events by the given handler,
	 * unless an error condition occurs.
	 */
	ERR (TLCALL *processHalEvents)(HalEventHandler handler, PTR secondArgToHandler);
	UNK (TLCALL *getZero_funcC)();
	BYTE (TLCALL *getKeyImpliedRunLevel)(void);
	ERR (TLCALL *resetWDogCheckPowerDown)(void);
	ERR (TLCALL *launchExternalDosProgram)();
	ERR (TLCALL *formatCard)();
	void (TLCALL *setBeepEnable)(BOOL enable);
	BOOL (TLCALL *isBeepEnabled)(void);
	void (TLCALL *setSpecialExtension)(char* extension);

	/*
	 * Buffer must be 4 bytes for the zero terminated
	 * 3 character extension. If the stored extension
	 * is not only letters and digits, or if it is "THM",
	 * a zero length string will be returned.
	 */
	void (TLCALL *getSpecialExtension)(char* extensionBuf);
	void (TLCALL *setOwnerName)(char* name);
	char* (TLCALL *getOwnerName)(void);
	void (TLCALL *setSomePrefDWord)(DWORD value);
	DWORD (TLCALL *getSomePrefDWord)(void);
	WORD (TLCALL *getRunStateFlags)(void);
	void (TLCALL *setLastErrorCode)(ERR errorCode);
	void (TLCALL *setPrefPackByte2_03)(void);
	void (TLCALL *setCurrentLangID)(BYTE langID);
	BYTE (TLCALL *getCurrentLangID)(void);
	ERR (TLCALL *savePrefsToNvm)(void);
	DWORD dummy58[4];
	
	/*
	 * NOTE: Since you cannot pass any extra args,
	 * do not include any formatting "%" chars in the string.
	 * Also, destination may vary, but may be stdout.
	 */
	void (TLCALL *debugPrintf)(char* string);
	DWORD dummy6C[4];
	UNK (TLCALL *func7C)();
	UNK (TLCALL *func80)();
	UNK (TLCALL *func84)();
	ERR (TLCALL *copyUsbStructIntoFreeSlot)(void* usbStruct);
	UNK (TLCALL *func8C)();
	UNK (TLCALL *func90)();
	void (TLCALL *unlinkOnlyUsbSlot)(void* usbStruct);
	void (TLCALL *freeUsbSlotAndPayload)(void* usbStruct);
	UNK (TLCALL *func9C)();
	UNK (TLCALL *funcA0)();
	DWORD dummyA4;
	UNK (TLCALL *funcA8)();
	DWORD (TLCALL *getBattTimeLeftEtc)(void);
	UNK (TLCALL *funcB0)();
	void (TLCALL *getCurrentTimeFromHW)(PackedTimeStruct* time);
	void (TLCALL *setCurrentTimeFromHW)(PackedTimeStruct* time);
	ulong (TLCALL *getTimerTicks)(void);
	UNK (TLCALL *selectSubFor0_7600)(BOOL type);
	UNK (TLCALL *funcC4)();
	WORD (TLCALL *getProductId)(void);
};

#define SYS (*((struct TearlessSysLib*)(TEARLESS_LIBS[1])))

#pragma pack (pop);
#ifdef __cplusplus
}
#endif

#endif

