/*************************************************************************
 * Copyright (c) 2004 Nick T Tate
 * 
 * This file is part of Tearless SDK.
 * 
 * Tearless SDK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Tearless SDK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Tearless SDK; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 ************************************************************************/
/*
 * Header file for Tearless Widget library (Lib 10).
 *
 * Author: Nick T Tate 2003-09-07
 * 
 * $Id: tlwidget.h 12 2004-10-21 11:51:11Z nicktt $
 */

#ifndef TEARLESS_WIDGET_HEADER
#define TEARLESS_WIDGET_HEADER

#include "tlcommon.h"

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack (push, 1);


typedef struct ProgressBar {
	WORD reserved_0;
	ushort divisor;
	WORD reserved_4;
	ushort x, y;
	ushort width, height;

#ifdef TEARLESS_USE_CLASSES
	void draw(void);
#endif
} ProgressBar;

typedef struct Line {
	BOOL shadowed;
	ushort x, y;
	ushort width, height;

#ifdef TEARLESS_USE_CLASSES
	void draw(void);
#endif
} Line;

typedef enum WidgetStatus {
	STATUS_UNSELECTED = 0,
	STATUS_SELECTED = 1,
	// Maybe
	STATUS_CHECKED = 2,
	STATUS_DISABLED = 3,
	// Maybe
	STATUS_TRANSPARENT = 4,
	STATUS_5 = 5,
	// Maybe
	STATUS_CLEAR = 6,
	STATUS_7 = 7
} WidgetStatus;

typedef enum WidgetStyle {
	// Border colors are (in status order): 7, 6, 5
	STYLE_BORDER = 0,
	// Direction and colors are: UP4, UP3, UP7, DOWN5
	STYLE_BOX_3D = 1,
	STYLE_NO_BG_2 = 2,
	STYLE_NO_BG_3 = 3,
	// Direction and colors are: FLAT6, DOWN5, FLAT6
	STYLE_BOX = 4,
	// Circle colors are: 7, 5 (Only valid for icon)
	STYLE_ICON_ON_CIRCLE = 5,
	STYLE_NO_BG_6 = 6,
	STYLE_NO_BG_7 = 7,
	// Colors are: 8, E, C (Only valid for icon)
	STYLE_ICON_ALT = 8
} WidgetStyle;


typedef enum WidgetType {
	// Foreground colors are: B, 1, D, D, 7, D, 7, E
	WIDGET_PLAIN = 0,
	// Foreground colors are: B, B, D, B
	WIDGET_FANCY = 1,
	WIDGET_TAB = 2,
	WIDGET_TEXT_SPIN = 0x43,
	WIDGET_INT_SPIN = 0x44,
	WIDGET_45 = 0x45
} WidgetType;

typedef struct PlainWidget {
	BYTE status;
	BOOL shadowText;
	BOOL isImageNumNotText;
	ushort x, y;
	PTR data;

#ifdef TEARLESS_USE_CLASSES
	void draw(void);
#endif
} PlainWidget;

typedef struct FancyWidget {
	BYTE status;	// WidgetStatus
	BYTE style;		// WidgetStyle
	BOOL isImageNumNotText;
	ushort x, y;
	ushort leftPadding;
	ushort width, height;
	COLOR reservedColor;
	PTR data;

#ifdef TEARLESS_USE_CLASSES
	void draw(void);
#endif
} FancyWidget;

typedef struct TabWidget {
	BYTE status;	// WidgetStatus
	BYTE pad_1;
	ushort x, y;
	ushort width, height;
	COLOR bgColor;
	ushort* imageNumPtr;

#ifdef TEARLESS_USE_CLASSES
	void draw(void);
	void fill(void);
#endif
} TabWidget;

typedef struct TextSpinner {
	BYTE status;	// WidgetStatus
	BOOL sunken3d;
	BYTE disableText;
	BYTE numTexts;
	ubyte currentIndex;
	ushort x, y;
	ushort width, height;
	BYTE pad_D;
	char** valueTexts;
	WORD reserved_12;

#ifdef TEARLESS_USE_CLASSES
	void draw(void);
	void prevDraw(void);
	void nextDraw(void);
#endif
} TextSpinner;

typedef struct IntSpinner {
	BYTE status;	// WidgetStatus
	BOOL sunken3d;
	BYTE numDigits;
	BOOL uninitialized;
	ushort value;
	ushort maxValue, minValue;
	ushort x, y;
	ushort width, height;

#ifdef TEARLESS_USE_CLASSES
	void decDraw(void);
	void incDraw(void);
#endif
} IntSpinner;

// TLS100: Broken
typedef struct Widget45 {
	BYTE status;	// WidgetStatus
	BYTE is10or11;
	BYTE upToTwo;
	BYTE pad_3;
	ushort x, y;

#ifdef TEARLESS_USE_CLASSES
	void draw(void);
	void do1andDraw(void);
	void do2andDraw(void);
#endif
} Widget45;

typedef union Widget {
	BYTE status;	// WidgetStatus
	PlainWidget plain;
	FancyWidget fancy;
	TabWidget tab;
	TextSpinner textSpin;
	IntSpinner intSpin;
	Widget45 widget45;
} Widget;

// Shape lib
struct TearlessWidgetLib {
	void (TLCALL *drawProgressBar)(ProgressBar* bar);
	void (TLCALL *drawFancyWidget)(FancyWidget* widget);
	void (TLCALL *drawTabWidget)(TabWidget* widget);
	void (TLCALL *fillTabWidget)(TabWidget* widget);
	void (TLCALL *drawTextSpinner)(TextSpinner* spinner);
	void (TLCALL *prevDrawTextSpinner)(TextSpinner* spinner);
	void (TLCALL *nextDrawTextSpinner)(TextSpinner* spinner);
	void (TLCALL *drawPlainWidget)(PlainWidget* widget);
	void (TLCALL *drawHorizLine)(Line* line);
	void (TLCALL *decDrawIntSpinner)(IntSpinner* spinner);
	void (TLCALL *incDrawIntSpinner)(IntSpinner* spinner);
	void (TLCALL *drawWidget45)(Widget45* widget);
	void (TLCALL *do1andDrawWidget45)(Widget45* widget);
	void (TLCALL *do2andDrawWidget45)(Widget45* widget);
	void (TLCALL *draw3DRect)(WORD a1, WORD a2, WORD a3, WORD a4, WORD a5, WORD a6);
	void (TLCALL *drawRectBorder)(ushort x, ushort y, ushort width, ushort height, BOOL colorStyle);
	void (TLCALL *func40)();
	void (TLCALL *func44)();
	void (TLCALL *drawImageOf4At)();
	void (TLCALL *drawImageOf2At)();
};

#define WIDGET (*((struct TearlessWidgetLib*)(TEARLESS_LIBS[10])))

#ifdef TEARLESS_USE_CLASSES
/*
 * Inline wrappers for instance methods defined in library.
 */

	// ProgressBar
	inline void ProgressBar::draw(void) { WIDGET.drawProgressBar(this); };

	// Line
	inline void Line::draw(void) { WIDGET.drawHorizLine(this); };

	// PlainWidget
	inline void PlainWidget::draw(void) { WIDGET.drawPlainWidget(this); };

	// FancyWidget
	inline void FancyWidget::draw(void) { WIDGET.drawFancyWidget(this); };

	// TabWidget
	inline void TabWidget::draw(void) { WIDGET.drawTabWidget(this); };
	inline void TabWidget::fill(void) { WIDGET.fillTabWidget(this); };

	// TextSpinner
	inline void TextSpinner::draw(void) { WIDGET.drawTextSpinner(this); };
	inline void TextSpinner::prevDraw(void) { WIDGET.prevDrawTextSpinner(this); };
	inline void TextSpinner::nextDraw(void) { WIDGET.nextDrawTextSpinner(this); };

	// IntSpinner
	inline void IntSpinner::decDraw(void) { WIDGET.decDrawIntSpinner(this); };
	inline void IntSpinner::incDraw(void) { WIDGET.incDrawIntSpinner(this); };

	// Widget45
	inline void Widget45::draw(void) { WIDGET.drawWidget45(this); };
	inline void Widget45::do1andDraw(void) { WIDGET.do1andDrawWidget45(this); };
	inline void Widget45::do2andDraw(void) { WIDGET.do2andDrawWidget45(this); };
#endif

#pragma pack (pop);
#ifdef __cplusplus
}
#endif

#endif

